@@ -19,14 +19,13 @@ Note, that all annotated challenges act as a proxy objects::
 """
 import logging
 
-from acme import challenges
-from acme import jose
+import josepy as jose
 
+from acme import challenges
 
 logger = logging.getLogger(__name__)
 
 
-# pylint: disable=too-few-public-methods
 
 class AnnotatedChallenge(jose.ImmutableMap):
     """Client annotated challenge.
