@@ -1,8 +1,9 @@
 """Base test class for DNS authenticators built on Lexicon."""
 
+import josepy as jose
 import mock
-from acme import jose
-from requests.exceptions import HTTPError, RequestException
+from requests.exceptions import HTTPError
+from requests.exceptions import RequestException
 
 from certbot import errors
 from certbot.plugins import dns_test_common
