@@ -1,32 +1,29 @@
-"""Test utilities.
-
-.. warning:: This module is not part of the public API.
-
-"""
+"""Test utilities."""
 import logging
+from multiprocessing import Event
+from multiprocessing import Process
 import shutil
 import sys
 import tempfile
 import unittest
-from multiprocessing import Process, Event
 
-import OpenSSL
-import josepy as jose
-import mock
-import pkg_resources
-import six
-from six.moves import reload_module  # pylint: disable=import-error
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import serialization
+import josepy as jose
+import mock
+import OpenSSL
+import pkg_resources
+import six
+from six.moves import reload_module
 
-from certbot import configuration
-from certbot import constants
 from certbot import interfaces
-from certbot import lock
-from certbot import storage
 from certbot import util
-from certbot.compat import os
+from certbot._internal import configuration
+from certbot._internal import constants
+from certbot._internal import lock
+from certbot._internal import storage
 from certbot.compat import filesystem
+from certbot.compat import os
 from certbot.display import util as display_util
 
 
@@ -56,8 +53,7 @@ def _guess_loader(filename, loader_pem, loader_der):
         return loader_pem
     elif ext.lower() == '.der':
         return loader_der
-    else:  # pragma: no cover
-        raise ValueError("Loader could not be recognized based on extension")
+    raise ValueError("Loader could not be recognized based on extension")  # pragma: no cover
 
 
 def load_cert(*names):
@@ -94,26 +90,6 @@ def load_pyopenssl_private_key(*names):
     return OpenSSL.crypto.load_privatekey(loader, load_vector(*names))
 
 
-def skip_unless(condition, reason):  # pragma: no cover
-    """Skip tests unless a condition holds.
-
-    This implements the basic functionality of unittest.skipUnless
-    which is only available on Python 2.7+.
-
-    :param bool condition: If ``False``, the test will be skipped
-    :param str reason: the reason for skipping the test
-
-    :rtype: callable
-    :returns: decorator that hides tests unless condition is ``True``
-
-    """
-    if hasattr(unittest, "skipUnless"):
-        return unittest.skipUnless(condition, reason)
-    elif condition:
-        return lambda cls: cls
-    return lambda cls: None
-
-
 def make_lineage(config_dir, testfile):
     """Creates a lineage defined by testfile.
 
@@ -254,8 +230,7 @@ class FreezableMock(object):
         if self._frozen:
             if name in self._frozen_set:
                 raise AttributeError('Cannot change frozen attribute ' + name)
-            else:
-                return setattr(self._mock, name, value)
+            return setattr(self._mock, name, value)
 
         if name != '_frozen_set':
             self._frozen_set.add(name)
@@ -269,7 +244,7 @@ class FreezableMock(object):
 def _create_get_utility_mock():
     display = FreezableMock()
     # Use pylint code for disable to keep on single line under line length limit
-    for name in interfaces.IDisplay.names():  # pylint: disable=no-member,E1120
+    for name in interfaces.IDisplay.names():  # pylint: E1120
         if name != 'notification':
             frozen_mock = FreezableMock(frozen=True, func=_assert_valid_call)
             setattr(display, name, frozen_mock)
@@ -294,7 +269,7 @@ def _create_get_utility_mock_with_stdout(stdout):
 
     display = FreezableMock()
     # Use pylint code for disable to keep on single line under line length limit
-    for name in interfaces.IDisplay.names():  # pylint: disable=no-member,E1120
+    for name in interfaces.IDisplay.names():  # pylint: E1120
         if name == 'notification':
             frozen_mock = FreezableMock(frozen=True,
                                         func=_write_msg)
